summaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorAdriaan Moors <adriaan.moors@epfl.ch>2010-05-04 13:53:09 +0000
committerAdriaan Moors <adriaan.moors@epfl.ch>2010-05-04 13:53:09 +0000
commit7abeacab53286233d4b150e713ca253c08f38bfd (patch)
treee6f75af92ef5602d9170ba1ad91df20b20111b4c /src
parent6bc86b824867b9c0eaab79bcd798d05759a9167a (diff)
downloadscala-7abeacab53286233d4b150e713ca253c08f38bfd.tar.gz
scala-7abeacab53286233d4b150e713ca253c08f38bfd.tar.bz2
scala-7abeacab53286233d4b150e713ca253c08f38bfd.zip
closes #3373, #3177: validity check of an impli...
closes #3373, #3177: validity check of an implicit value should consider the value as well as its accessor review by odersky
Diffstat (limited to 'src')
-rw-r--r--src/compiler/scala/tools/nsc/typechecker/Implicits.scala14
1 files changed, 10 insertions, 4 deletions
diff --git a/src/compiler/scala/tools/nsc/typechecker/Implicits.scala b/src/compiler/scala/tools/nsc/typechecker/Implicits.scala
index 9a6c4cc401..862f9adaec 100644
--- a/src/compiler/scala/tools/nsc/typechecker/Implicits.scala
+++ b/src/compiler/scala/tools/nsc/typechecker/Implicits.scala
@@ -520,7 +520,7 @@ self: Analyzer =>
* - the symbol's type is initialized
* - the symbol comes from a classfile
* - the symbol comes from a different sourcefile than the current one
* - the symbol's definition comes before, and does not contain the closest enclosing definition,
* - the symbol's definition is a val, var, or def with an explicit result type
* The aim of this method is to prevent premature cyclic reference errors
* by computing the types of only those implicits for which one of these
@@ -539,9 +539,15 @@ self: Analyzer =>
case _ => true
}
}
- def comesBefore(sym: Symbol, owner: Symbol) =
- sym.pos.pointOrElse(0) < owner.pos.pointOrElse(Integer.MAX_VALUE) &&
- !(owner.ownerChain contains sym)
+ def comesBefore(sym: Symbol, owner: Symbol) = {
+ val ownerPos = owner.pos.pointOrElse(Integer.MAX_VALUE)
+ sym.pos.pointOrElse(0) < ownerPos &&
+ if(sym isGetterOrSetter) {
+ val symAcc = sym.accessed // #3373
+ symAcc.pos.pointOrElse(0) < ownerPos &&
+ !(owner.ownerChain exists (o => (o eq sym) || (o eq symAcc))) // probably faster to iterate only once, don't feel like duplicating hasTransOwner for this case
+ } else !(owner hasTransOwner sym) // faster than owner.ownerChain contains sym
+ }
sym.isInitialized ||
sym.sourceFile == null ||
@@ -520,7 +520,7 @@ self: Analyzer =>
* - the symbol comes from a classfile
* - the symbol comes from a different sourcefile than the current one
- * - the symbol's definition comes before, and does not contain the closest enclosing definition,
+ * - the symbol and the accessed symbol's definitions come before, and do not contain the closest enclosing definition, // see #3373
* - the symbol's definition is a val, var, or def with an explicit result type
* The aim of this method is to prevent premature cyclic reference errors